import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A rocket that can be controlled by the arrowkeys: up, left, right.
 * The gun is fired by hitting the 'space' key. 'z' releases a proton wave.
 * 
 * @author Poul Henriksen
 * @author Michael Kolling
 * 
 * @version 1.0
 */
public class Rocket extends SmoothMover
{
    private static final int gunReloadTime = 5;         // The minimum delay between firing the gun.
    private static final int protonWaveReloadTime = 200;         // The minimum delay between creating proton waves.
    private static final int shieldTime = 350;         // How much time does the shield work.
    private static final int almostVisibleShieldBegins = 250;         // When do we change the color of the shield after it turned on. 

    private int reloadDelayCount;               // How long ago we fired the gun the last time.
    private int reloadDelayCountProtonWave;               // How long ago we created a proton wave the last time.
    private int shieldCount;               // How long ago we turned on the sield.
    
    private boolean shieldMode = false; // Is the shield on?
    
    private GreenfootImage rocket = new GreenfootImage("rocket.png");    
    private GreenfootImage rocketWithThrust = new GreenfootImage("rocketWithThrust.png");
    private boolean boosterWorked = false;

    /**
     * Initilise this rocket.
     */
    public Rocket()
    {
        reloadDelayCount = gunReloadTime;
        reloadDelayCountProtonWave = protonWaveReloadTime;
        //setRotation( Greenfoot.getRandomNumber(359) );
        Vector force = new Vector(getRotation(), 1.0);
        addForce(force);
    }

    /**
     * Do what a rocket's gotta do. (Which is: mostly flying about, and turning,
     * accelerating and shooting when the right keys are pressed.)
     */
    public void act()
    {
        checkKeys();
        reloadDelayCount++;
        reloadDelayCountProtonWave++;
        checkShieldCount();
        move();
        if(!shieldMode) checkCollision();
    }
    
    /**
     * Check whether there are any key pressed and react to them.
     */
    private void checkKeys() 
    {
        if (Greenfoot.isKeyDown("space")) 
        {
            fire();
        }
        if( Greenfoot.isKeyDown("z") ) {
           startProtonWave();
        }
        
        if (Greenfoot.isKeyDown("left")) 
        {
            setRotation(getRotation()-5);
        }
        if (Greenfoot.isKeyDown("right")) 
        {
            setRotation(getRotation()+5);
        }
        
        if( Greenfoot.isKeyDown("up") ) ignite( getRotation() );
        else if( Greenfoot.isKeyDown("down") ) ignite( getRotation()+180 );
        else {
          setImage(rocket);
          boosterWorked = false;
        }
    }
    
    /**
     * Fire a bullet if the gun is ready.
     */
    private void fire() 
    {
        if (reloadDelayCount >= gunReloadTime) 
        {
            Bullet bullet = new Bullet (getMovement().copy(), getRotation());
            getWorld().addObject (bullet, getX(), getY());
            bullet.move ();
            reloadDelayCount = 0;
        }
    }
    
    /**
     * Turn off the shield mode when the shield counter finish the counting.
     */
    public void checkShieldCount()
    {
       if(shieldMode && shieldCount == almostVisibleShieldBegins) {
          rocket = new GreenfootImage("almostVisibleRocket.png");
          rocketWithThrust = new GreenfootImage("almostVisibleRocketWithThrust.png");
          setImage(rocketWithThrust);
       }
       if(shieldMode && shieldCount == shieldTime) {
          setShield(false);
       }
       else if(shieldMode) shieldCount++;
    }
    
    /**
     * Increase the speed of the rocket and change its image when the given boolean is true.
     */
    public void ignite(int direction)
    {
       if(!boosterWorked) {
          setImage(rocketWithThrust);
          boosterWorked = true;
       }
       Vector force = new Vector(direction, 0.1 + 0.05 * ( ((Space) getWorld()).getLevel()-1 ));
       addForce(force);
    }
    
    private void checkCollision()
    {
       Actor a = getOneIntersectingObject(Asteroid.class);
       if( a != null ) {
          Space space = (Space) getWorld();
          GreenfootSound backgroundMusic = space.getBackgroundMusic();
          if(space.getLives() > 1) {
             //if(!((Space) getWorld()).muteButton.mute) backgroundMusic.pause();
             if(!((Space) getWorld()).muteButton.mute) Greenfoot.playSound("MetalExplosion.wav");
             Greenfoot.delay(30);
             space.addLives(-1);
             setLocation(getWorld().getWidth()/2 + 100, getWorld().getHeight()/2);
             setShield(true);
             Greenfoot.delay(30);
             //if(!((Space) getWorld()).muteButton.mute) backgroundMusic.playLoop();
          }
          else {
             //if(!((Space) getWorld()).muteButton.mute) backgroundMusic.pause();
             if(!((Space) getWorld()).muteButton.mute) Greenfoot.playSound("MetalExplosion.wav");
             space.addLives(-1);
             space.addObject(new Explosion(), getX(), getY());
             space.gameOver();
             space.removeObject(this);
             //if(!((Space) getWorld()).muteButton.mute) backgroundMusic.playLoop();
          }
       }
    }
    
    /**
     * Create a new proton wave and place it in the world.
     */
    public void startProtonWave()
    {
       if (reloadDelayCountProtonWave >= protonWaveReloadTime) 
       {
          getWorld().addObject( new ProtonWave(), getX(), getY() );
          if(!((Space) getWorld()).muteButton.mute) Greenfoot.playSound("proton.wav");
          reloadDelayCountProtonWave = 0;
       }
    }
    
    /**
     * Return how long ago we created a proton wave the last time.
     */
    public int timeToReloadProtonWave()
    {
       return reloadDelayCountProtonWave;
    }
    
    /**
     * Turn on/off the shield mode and set the image of the rocket.
     */
    public void setShield()
    {
       if(!shieldMode) {
          rocket = new GreenfootImage("invisibleRocket.png");
          rocketWithThrust = new GreenfootImage("invisibleRocketWithThrust.png");
          setImage(rocket);
          shieldCount = 0;
          shieldMode = true;
       }
       else {
          rocket = new GreenfootImage("rocket.png");
          rocketWithThrust = new GreenfootImage("rocketWithThrust.png");
          setImage(rocket);
          shieldMode = false;
       }
    }
    /**
     * Turn on/off the shield mode by a given boolean and set the image of the rocket.
     */
    public void setShield(boolean turnOn)
    {
       if(turnOn) {
          rocket = new GreenfootImage("invisibleRocket.png");
          rocketWithThrust = new GreenfootImage("invisibleRocketWithThrust.png");
          setImage(rocket);
          shieldCount = 0;
          shieldMode = true;
       }
       else {
          rocket = new GreenfootImage("rocket.png");
          rocketWithThrust = new GreenfootImage("rocketWithThrust.png");
          setImage(rocket);
          shieldMode = false;
       }
    }
    
    /**
     * Set the direction of the rocket by a given int.
     */
    public void setRocketRotation(int d)
    {
       setRotation(d);
    }
}